package org.netkernel.rdbms.endpoint;

import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.rdbms.util.RDBMSUtil;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public abstract class AbstractEscapeAccessor extends StandardAccessorImpl
{

	protected void recurseEscapeHDS(HDSBuilder builder, IHDSNode node)
	{	Object o=node.getValue();
		if (o!=null && o instanceof String)
		{	o=escape((String) o);
		}
		boolean pflag=node.getParent()!=null;
		if (pflag)
		{	builder.pushNode(node.getName(), o);
		}
		IHDSNode[] children=node.getChildren();
		for (IHDSNode child : children)
		{	recurseEscapeHDS(builder, child);
		}
		if (pflag)
		{	builder.popNode();
		}
	}

	protected void recurseEscapeDOM(Node node)
	{
		if (node.getNodeType()==Node.TEXT_NODE)
		{	node.setTextContent(escape(node.getTextContent()));
		}
		else if (node.getNodeType()==Node.ELEMENT_NODE)
		{	NamedNodeMap map=node.getAttributes();
			for (int i=0; i<map.getLength(); i++)
			{	Node m=map.item(i);
				m.setTextContent(escape(m.getTextContent()));
			}
		}
		if (node.hasChildNodes())
		{	NodeList nodeList=node.getChildNodes();
			for (int i=0; i<nodeList.getLength(); i++)
			{	Node m=nodeList.item(i);
				recurseEscapeDOM(m);
			}
		}
	}


	protected String escape(String s)
	{	return RDBMSUtil.inEscape(s);
	}

}
