/******************************************************************************
 (c) Copyright 2002 - $Date: 2008/04/30 13:29:48 $ 1060 Research Ltd

 This Software is licensed to You, the licensee, for use under the terms of
 the 1060 Public License v1.0. Please read and agree to the 1060 Public
 License v1.0 [www.1060research.com/license] before using or redistributing
 this software.

 In summary the 1060 Public license has the following conditions.
 A. You may use the Software free of charge provided you agree to the terms
 laid out in the 1060 Public License v1.0
 B. You are only permitted to use the Software with components or applications
 that provide you with OSI Certified Open Source Code [www.opensource.org], or
 for which licensing has been approved by 1060 Research Limited.
 You may write your own software for execution by this Software provided any
 distribution of your software with this Software complies with terms set out
 in section 2 of the 1060 Public License v1.0
 C. You may redistribute the Software provided you comply with the terms of
 the 1060 Public License v1.0 and that no warranty is implied or given.
 D. If you find you are unable to comply with this license you may seek to
 obtain an alternative license from 1060 Research Limited by contacting
 license@1060research.com or by visiting www.1060research.com

 NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 LICENSE V1.0 FOR DETAILS

 THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license

 File:          $RCSfile: RDBMSAccessor.java,v $
 Version:       $Name:  $ $Revision: 1.2 $
 Last Modified: $Date: 2008/04/30 13:29:48 $
 *****************************************************************************/

package org.netkernel.rdbms.endpoint;


import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer1.representation.IDeterminateStringRepresentation;
import org.netkernel.rdbms.representation.IAspectDBConnectionPool;
import org.netkernel.rdbms.util.RDBMSUtil;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.util.TimerTask;

public class SQLProcAccessor extends AbstractRDBMSAccessor
{

	public SQLProcAccessor()
	{	this.declareThreadSafe();
	}

	public void onSource(INKFRequestContext context) throws Exception
	{	INKFResponse response;

		String configURI;
		String sqlQuery;
		Object resultant;

		configURI=this.getConfigURI(context);
		sqlQuery=context.source("arg:operand", IDeterminateStringRepresentation.class).getString();

		IAspectDBConnectionPool connectionPool=context.source(configURI, IAspectDBConnectionPool.class);
		Connection connection=connectionPool.acquireConnection();
		try
		{	RDBMSUtil.setContextClassloader(context);
			CallableStatement callableStatement=connection.prepareCall(sqlQuery);
			TimerTask timerTask=configureStatementTimeout(callableStatement, connectionPool);
			try
			{	boolean hasResultSet=callableStatement.execute();
				HDSBuilder builder=new HDSBuilder();
				builder.pushNode("resultsets");
				while (true)
				{	if (!hasResultSet)
					{	int updateCount=callableStatement.getUpdateCount();
						if (updateCount==-1)
						{	break;
						}
					}
					else
					{	ResultSet resultSet=callableStatement.getResultSet();
						RDBMSUtil.resultsetToHDS(builder, resultSet);
						resultSet.close();
					}
					if (!callableStatement.getMoreResults())
					{	break;
					}
				}
				builder.popNode();
				resultant=builder.getRoot();
			}
			finally
			{	if (timerTask!=null)
				{	timerTask.cancel();
				}
				callableStatement.close();
			}
			response=context.createResponseFrom(resultant);
		}
		catch (Exception e)
		{	response=context.createResponseFrom(e);
		}
		finally
		{	connectionPool.releaseConnection(connection);
		}
		response.setNoCache();
	}
}
