/******************************************************************************
 (c) Copyright 2002 - $Date: 2008/04/11 11:57:59 $ 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: ResultSetRep.java,v $
 * Version:       $Name:  $ $Revision: 1.1 $
 * Last Modified: $Date: 2008/04/11 11:57:59 $
 *****************************************************************************/
package org.netkernel.rdbms.representation;


import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.rdbms.endpoint.AbstractRDBMSAccessor;
import org.netkernel.urii.INetKernelException;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.TimerTask;

public class ResultSetRepresentation
{	private String mQuery;
	private IAspectDBConnectionPool mPool;
	private ResultSet mResultSet;
	private AbstractRDBMSAccessor mAccessor;

	/**
	 * Creates a new instance of ResultSetAspect
	 */
	public ResultSetRepresentation(String aQuery, IAspectDBConnectionPool aPool, AbstractRDBMSAccessor aAccessor)
	{	mQuery=aQuery;
		mAccessor=aAccessor;
		mPool=aPool;
	}


	/**
	 * Returns a fresh ResultSet.
	 * This method may throw an NKFExceptionblock if result set is currently in use.
	 *
	 * @throws SQLException thrown if connection to db fails or query is invalid
	 * @throws NKFException thrown if result set is currently in use or if db configuration not found
	 */
	public synchronized ResultSet getResultSet() throws SQLException, NKFException
	{	if (mResultSet!=null)
		{	throw new NKFException("Concurrent ResultSet Access", "An org.ten60.rdbms.representation.ResultSetRepresentation must be released before reusing", mQuery, null);
		}
		Connection connection=null;
		Statement statement=null;
		TimerTask timerTask=null;
		try
		{	connection=mPool.acquireConnection();
			statement=connection.createStatement();
			timerTask=mAccessor.configureStatementTimeout(statement, mPool);
			statement.setQueryTimeout(mPool.getDefaultQueryTimeout());
			mResultSet=statement.executeQuery(mQuery);
			return mResultSet;
		}
		catch (SQLException e)
		{	if (statement!=null)
			{	statement.close();
			}
			if (connection!=null)
			{	mPool.releaseConnection(connection);
			}
			throw e;
		}
		catch (INetKernelException e)
		{
			NKFException e2=new NKFException(e.getId(), e.getMessage(), null);
			if (e.getCause()!=null)
			{	//e2.addCause(e.getCause());
			}
			throw e2;
		}
		finally
		{	if (timerTask!=null)
			{	timerTask.cancel();
			}
		}
	}

	/**
	 * release result set for use by another
	 */
	public synchronized void release()
	{	if (mResultSet!=null)
		{	Statement s=null;
			try
			{	s=mResultSet.getStatement();
				mResultSet.close();
			}
			catch (SQLException e)
			{ /* silently ignore */ }
			Connection c=null;
			try
			{	if (s!=null)
				{	c=s.getConnection();
					s.close();
				}
			}
			catch (SQLException e)
			{ /* silently ignore */ }
			if (c!=null)
			{	mPool.releaseConnection(c);
			}
		}
		mResultSet=null;
	}
}
